
function [hat_X,P] = kalman(Y,F,G,H,Q,R,S,Pi_0,hat_x_0)


% This routine  implements the Kalman filter

% The system is assumed to be time-invariant 
% and all observations are passed into the 
% function in a single matrix Y.

% The model is assumed to be of the form
% x_{i+1} = F x_i + G u_i
%   y_{i} = H x_i + v_i

% F is n x n
% G is n x m
% H is p x n
% Q is m x m
% R is p x p
% S is m x p

% Pi_0 is n x n positive-definite

% hat_x_0 is n x 1 initial condition (usually zero)

% Y is p x N is a matrix with ALL  N observations, each
% of dimension p x 1. That is, each column of Y is an
% observation vector

% hat_X is n x (N+1). Each column contains the resulting
% n x 1 state estimate [up to hat{x}_{N+1}].

% P is n x n and it contains the value of the Riccati
% variable at the end of the iterations (approx. steady-state)

% Written by A. H. Sayed, Dec. 6, 1999




[p,N]=size(Y);
P=Pi_0;
hat_X(:,1)=hat_x_0;

for i=1:N
 R_e=R+H*P*H';
 Ka=(F*P*H'+G*S)*inv(R_e);
 hat_X(:,i+1)=F*hat_X(:,i)+Ka*(Y(:,i)-H*hat_X(:,i));
 P=F*P*F'+G*Q*G'-Ka*R_e*Ka';
end
